//
//  MedtechDetectorConstruction.cc
//  medtech
//
//  Created by Baranyai David on 2018. 04. 01..
//

#include "MedtechDetectorConstruction.hh"

MedtechDetectorConstruction::MedtechDetectorConstruction() : G4VUserDetectorConstruction()
{
    
}

MedtechDetectorConstruction::~MedtechDetectorConstruction()
{
    
}

G4VPhysicalVolume* MedtechDetectorConstruction::Construct()
{
    Parameters *parameter = Parameters::getInstance();
    
    //Material Section
    G4NistManager *manager = G4NistManager::Instance();
    G4Material *air = manager -> FindOrBuildMaterial("G4_AIR");
    G4Material *targetMaterial = manager -> FindOrBuildMaterial("G4_Pb");
    G4Material *boxMaterial = manager -> FindOrBuildMaterial("G4_W");
    G4Material *coneMaterial = manager -> FindOrBuildMaterial("G4_Fe");
    G4Material *waterMaterial = manager -> FindOrBuildMaterial("G4_WATER");
    //End of Material Section
    
    //Physical world
    G4int world_xyz = 1*m; // world is 1m x 1m x 2m for each
    G4Box *solidWorld = new G4Box("World",world_xyz,world_xyz,4*m); //define a box for world
    
    G4LogicalVolume *logicWorld = new G4LogicalVolume(solidWorld, air, "World", 0, 0, 0); //define a logical volume for world
    
    G4PVPlacement *physicalWorld = new G4PVPlacement(0, //no rotation
                                                     G4ThreeVector(), //at (0,0,0)
                                                     logicWorld, //it's logical volume
                                                     "World", //it's name
                                                     0, //it's mother volume
                                                     false, //no boolean operations
                                                     0); //no magnetic field
    //End of Physical world
    
    //Target box == screen
    G4Box *solidTarget = new G4Box("Target", 1*m, 1*m, 0.1*cm);
    
    G4LogicalVolume *logicTarget = new G4LogicalVolume(solidTarget, targetMaterial, "Collimator", 0, 0, 0);
    
    G4PVPlacement *physicalTarget = new G4PVPlacement(0, //no rotation
                                                      G4ThreeVector(0, 0, 1.1*m), //at (0,0,0)
                                                      logicTarget, //it's logical volume
                                                      "Target", //it's name
                                                      logicWorld, //it's mother volume
                                                      false, //no boolean operations
                                                      0); //no particular field
    //End of Target box
    
    //Water after screen
    G4Box *solidWater = new G4Box("Water", 1*m, 1*m, 1*cm);
    G4LogicalVolume *logicWater = new G4LogicalVolume(solidWater, waterMaterial, "Water", 0, 0, 0);
    G4PVPlacement *physicalWater = new G4PVPlacement(0, //no rotation
                                                      G4ThreeVector(0, 0, 1.2*m), //at (0,0,0)
                                                      logicWater, //it's logical volume
                                                      "Water", //it's name
                                                      logicWorld, //it's mother volume
                                                      false, //no boolean operations
                                                      0); //no particular field
    //Water after screen cells (parameterisation)
    /*G4Box *solidWaterCell = new G4Box("Water", 1*m, 1*m, 1*cm);
    G4VPhysicalVolume *logicWaterCell = new G4VPhysicalVolume(solidWaterCell, waterMaterial, "cellWater", 0, 0, 0);
    G4VPVParameterisation waterCell = new G4VPVParameterisation("cellWater", logicWaterCell, physicalWater, logicWaterCell, false, 0, true);
    */
    //End of water after screen
    
    //Collimator
    G4double box_size = parameter -> GetBoxSize();
    G4Box *solidB1 = new G4Box("B1", 1*m, 1*m, 0.1*cm);
    G4Box *solidB2 = new G4Box("B2", box_size*cm, box_size*cm, 1.5*cm);
    
    G4SubtractionSolid *Box = new G4SubtractionSolid("Box", solidB1, solidB2);
    G4LogicalVolume *logicBox = new G4LogicalVolume(Box, boxMaterial, "Box", 0, 0, 0);
    G4PVPlacement *physicalBox = new G4PVPlacement(0, //no rotation
                                                   G4ThreeVector(0, 0, 10*cm),
                                                   logicBox,
                                                   "Box",
                                                   logicWorld,
                                                   false,
                                                   0);
    //End of Collimator
    
    //Cone
    G4double radius = parameter -> GetRadius();
    G4double tube_size = parameter -> GetTubeSize();
    G4double cone_size = parameter -> GetConeSize();
    G4double move_z = 0; //place the first solid to the right place
    G4double cone_placement = 0; //move cone to V(1,0,0)
    
    G4Cons *solidCone = 0;
    G4Tubs *solidTube = 0;
    G4UnionSolid *unionCone = 0;
    G4LogicalVolume *logicCone = 0;
    
    if(cone_size > 0 && tube_size != 0)
    {
        move_z = tube_size + cone_size;
        cone_placement = 2*tube_size + cone_size+ 1;
        
        solidCone = new G4Cons("cone", 0, radius*cm, 0, 0, cone_size*cm, 0, 2*CLHEP::pi); //name, inside R -Z, outside, R -Z, inside R Z, outside R Z, half length in z, vmi, vmi
        solidTube = new G4Tubs("tube", 0, radius*cm, tube_size*cm, 0, 2*CLHEP::pi); //name, inner R, outter R, Half length in Z, starting angle, angle of the segment in rad
        unionCone = new G4UnionSolid("cone", solidCone, solidTube, 0, G4ThreeVector(0, 0, -move_z*cm));
        logicCone = new G4LogicalVolume(unionCone, coneMaterial, "Cone", 0, 0, 0);
    }
    
    else if(cone_size == 0 && tube_size != 0)
    {
        cone_placement = tube_size + 1;
        solidTube = new G4Tubs("tube", 0, radius*cm, tube_size*cm, 0, 2*CLHEP::pi); //name, inner R, outter R, Half length in Z, starting angle, angle of the segment in rad
        logicCone = new G4LogicalVolume(solidTube, coneMaterial, "Cone", 0, 0, 0);
    }
    
    else if(cone_size < 0 && tube_size != 0)
    {
        if(cone_size > tube_size)
        {
            return 0;
        }
        else
        {
            G4RotationMatrix *rot = new G4RotationMatrix();
            rot -> rotateX(180 * deg);
            
            move_z = tube_size+cone_size;
            cone_placement = tube_size + 1;
            
            solidCone = new G4Cons("cone", 0, radius*cm, 0, 0, -cone_size*cm, 0, 2*CLHEP::pi); //name, inside R -Z, outside, R -Z, inside R Z, outside R Z, half length in z, vmi, vmi
            solidTube = new G4Tubs("tube", 0, radius*cm, tube_size*cm, 0, 2*CLHEP::pi); //name, inner R, outter R, Half length in Z, starting angle, angle of the segment in rad
            G4SubtractionSolid *subCone = new G4SubtractionSolid("Cone", solidTube, solidCone, rot, G4ThreeVector(0, 0, move_z*cm));
            logicCone = new G4LogicalVolume(subCone, coneMaterial, "Cone", 0, 0, 0);
        }
    }
    if(tube_size == 0)
    {
        cone_placement = cone_size + 1;
        solidCone = new G4Cons("cone", 0, radius*cm, 0, 0, cone_size*cm, 0, 2*CLHEP::pi); //name, inside R -Z, outside, R -Z, inside R Z, outside R Z, half length in z, vmi, vmi
        logicCone = new G4LogicalVolume(solidCone, coneMaterial, "Cone", 0, 0, 0);
    }
    
    G4PVPlacement *physicalCone = new G4PVPlacement(0,
                                                   G4ThreeVector(0, 0, cone_placement*cm),
                                                   logicCone,
                                                   "Cone",
                                                   logicWorld,
                                                   false,
                                                   0);
    
    return physicalWorld;
}
